clear all; close all; clc

%% načtení dat
load PPGset.mat 
load TFtab.mat
kvalita_ref = TFtab(:,7);  % seznam referenčních kvalit signálu
tepovka_ref = TFtab(:,4);  % seznam referenčních tepových frekvencí
ppg_signaly = PPGset(:,1);  % seznam PPG signálů z databáze

for x=1:100 %length(ppg_signaly)  %do 1962 ucho
    %% načtení daného signálu, referenční kvality a tepové frekvence
    ppg0 = ppg_signaly{x};
    HR_ann = tepovka_ref(x);
    kvalita_PPG_ann = kvalita_ref(x);

    %% předzpracování, stacionární vlnková transformace, detekce peaků a stanovení tepové frekvence
    ppg1 = ppg0 - mean(ppg0);  % odstranění stejnosměrné složky
    ppg_filt = lowpass(ppg1, 1.2, 30);  % filtrace signálu dolní propustí pro výpočet indexu perfuze
    ppg2 = wextend(1,'sp0',ppg1, 10); % umělé prodloužení signálu
    vlnkovka = swt(ppg2, 6, 'rbio2.4'); % stacionární vlnková transformace řádu 6
    vlnkovka = vlnkovka(:,11:310);  % zkrácení signálu na původních 300 vzorků
    ppg2 = vlnkovka(4,:);  % uložení 4. řádu
    [pks, locs] = findpeaks(ppg2,'MinPeakDistance', 10); % detekce peaků pro stanovení tepové frekvence
    
    if length(pks) < 2  % automatické zařazení signálu do skupiny špatné kvality pokud lze detekovat méně než 2 peaky
        kvalita_PPG = 0;
        HR = 0;
        podminka1 = 0;
        podminka2 = 0;
        pominka3 = 0;
        podminka4 = 0;
        podminka5 = 0;
    else

        PP = [];  % seznam pozic nalezených peaků
        
        % cyklus uloží pozice peaků
        for i=1:length(pks)-1;
            diff = locs(i+1) - locs(i);
            PP(i) = diff;
        end

        PP = median(PP);  % medián vzdálenosti peaků
        HR = 30/PP *60;  % výpočet tepové frekvence

        %% rozhodovací pravidlo 1 (HR mezi 60 a 130)
        if (HR >= 60) && (HR <= 130) 
            podminka1 = 1;
        else
            podminka1 = 0;
        end

        %% rozhodovací pravidlo 2 (RR intervaly menší než 2,2 s)
        RR = [];  % seznam peaků
        
        % cyklus pro uložení vzdálenosti peaků v sekundách a kontrolu
        % jestli žádný z nich nepřekročí interval 2,2 sekundy
        for i=1:length(pks)-1;
            diff = locs(i+1) - locs(i);
            diff = diff/30;
            RR(i) = diff;
            if RR(i) >= 2.2 | podminka1 == 0  
                podminka2 = 0;
                break
            else
                podminka2 = 1;
            end
        end

        %% rozhodovací pravidlo 3 (maximální RR interval / minimální RR interval menší než 4,2)
        if max(RR)/min(RR) < 4.2 & podminka2 == 1;  
            podminka3 = 1;
        else
            podminka3 = 0;
        end
        

        %% template matching
        medRR = median(RR)*30;  % rozdělení signálu na segmenty podle mediánové vzdálenosti peaků
        medRR = round(medRR);
        from = 1;
        to = medRR;
        
        % cyklus pro uložení jednotlivých segmentů
        for i=1:length(pks)
            seg{i} = ppg1(from:to);
            from = from + medRR;
            to = to + medRR;
            if to > 299
                break
            end
        end

        % výpočet vzoru PPG (šablony)
        prumernePPG = [];
        for i=1:length(seg{1})
            prumer = 0;
            for y=1:length(seg)
                signal = seg{y}(i);
                prumer = prumer + signal;
            end
            prumer = prumer/length(seg);
            prumernePPG(i) = prumer;
        end

        % výpočet korelačních koeficientů
        koeficienty = [];
        for i=1:length(seg)
            koef = corrcoef(prumernePPG, seg{i});
            koeficienty(i) = koef(1,2);
        end

        % kontrola jestli je průměrný korelační koeficient menší než práh
        % 0.17
        prum_koef = mean(koeficienty);
        if prum_koef >= 0.17 & podminka3 ==1   % 0.17
            podminka4 = 1;
        else
            podminka4 = 0;
        end      
        

        %% index P_SQI
        y_max = max(ppg_filt);  % maximální výchylka filtrovaného signálu
        y_min = min(ppg_filt);  % minimální výchylka filtrovaného signálu
        x_avg = mean(ppg0);  % průměr surového PPG signálu

        P_SQI =[(y_max-y_min)/abs(x_avg)] * 100;  % výpočet indexu perfuze

        % kontrola jestli je index menší než práh 25
        if P_SQI < 25 & podminka4 == 1 
            podminka5 = 1;
        else
            podminka5 = 0;
        end

    end
    
    
    %% Uložení výsledků

    % dobrá a špatná kvalita
    if podminka5 == 1
        kvalita_PPG = 1;
    else
        kvalita_PPG = 0;
    end
    
    % uložení všech výsledků
    vysledek(x,:) = {kvalita_PPG, kvalita_PPG_ann, podminka1, podminka2, podminka3, podminka4, ...
        podminka5, HR, HR_ann, P_SQI};
    clear seg
end

